# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CounterAdjustmentListParams"]


class CounterAdjustmentListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """List CounterAdjustment items for the Account UUID."""

    counter_id: Annotated[str, PropertyInfo(alias="counterId")]
    """List CounterAdjustment items for the Counter UUID."""

    date: str
    """List CounterAdjustment items for the given date."""

    date_end: Annotated[Optional[str], PropertyInfo(alias="dateEnd")]

    date_start: Annotated[Optional[str], PropertyInfo(alias="dateStart")]

    end_date_end: Annotated[str, PropertyInfo(alias="endDateEnd")]
    """Only include CounterAdjustments with end dates earlier than this date."""

    end_date_start: Annotated[str, PropertyInfo(alias="endDateStart")]
    """
    Only include CounterAdjustments with end dates equal to or later than this date.
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """nextToken for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of CounterAdjustments to retrieve per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["DestinationCreateParams"]


class DestinationCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    bucket_name: Required[Annotated[str, PropertyInfo(alias="bucketName")]]
    """Name of the S3 bucket for the Export Destination."""

    code: Required[str]
    """The code of the Export Destination."""

    iam_role_arn: Required[Annotated[str, PropertyInfo(alias="iamRoleArn")]]
    """
    To enable m3ter to upload a Data Exports to your S3 bucket, the service has to
    assume an IAM role with PutObject permission for the specified `bucketName`.
    Create a suitable IAM role in your AWS account and enter ARN:

    **Formatting Constraints:**

    - The IAM role ARN must begin with "arn:aws:iam".
    - The general format required is:
      "arn:aws:iam::<aws account id>:role/<role name>". For example:
      "arn:aws:iam::922609978421:role/IAMRole636".
    - If the `iamRoleArn` used doesn't comply with this format, then an error
      message will be returned.

    **More Details:** For more details and examples of the Permission and Trust
    Policies you can use to create the required IAM Role ARN, see
    [Creating Data Export Destinations](https://www.m3ter.com/docs/guides/data-exports/creating-data-export-destinations)
    in our main User documentation.
    """

    name: Required[str]
    """The name of the Export Destination."""

    partition_order: Annotated[Optional[Literal["TYPE_FIRST", "TIME_FIRST"]], PropertyInfo(alias="partitionOrder")]
    """
    Specify how you want the file path to be structured in your bucket destination -
    by Time first (Default) or Type first.

    Type is dependent on whether the Export is for Usage data or Operational data:

    - **Usage.** Type is `measurements`.
    - **Operational.** Type is one of the entities for which operational data
      exports are available, such as `account`, `commitment`, `meter`, and so on.

    Example for Usage Data Export using .CSV format:

    - Time first:
      `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    - Type first:
      `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    """

    prefix: str
    """Location in specified S3 bucket for the Export Destination.

    If you omit a `prefix`, then the root of the bucket will be used.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ScheduleListResponse"]


class ScheduleListResponse(BaseModel):
    id: str
    """The id of the Data Export Schedule."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    code: Optional[str] = None
    """Unique short code of the Data Export Schedule."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this Schedule."""

    destination_ids: Optional[List[str]] = FieldInfo(alias="destinationIds", default=None)
    """The Export Destination ids."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the Data Export Schedule was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the Schedule was last modified."""

    export_file_format: Optional[Literal["CSV", "JSON"]] = FieldInfo(alias="exportFileFormat", default=None)

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this Data Export Schedule."""

    name: Optional[str] = None
    """The name of the Data Export Schedule."""

    period: Optional[int] = None
    """Defines the Schedule frequency for the Data Export to run in Hours or Days.

    Used in conjunction with the `scheduleType` parameter.
    """

    schedule_type: Optional[Literal["HOURLY", "DAILY", "AD_HOC"]] = FieldInfo(alias="scheduleType", default=None)

    source_type: Optional[Literal["USAGE", "OPERATIONAL"]] = FieldInfo(alias="sourceType", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BalanceListParams"]


class BalanceListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """The unique identifier (UUID) for the end customer's account."""

    end_date_end: Annotated[str, PropertyInfo(alias="endDateEnd")]
    """Only include Balances with end dates earlier than this date."""

    end_date_start: Annotated[str, PropertyInfo(alias="endDateStart")]
    """Only include Balances with end dates equal to or later than this date."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for retrieving the next page of Balances.

    It is used to fetch the next page of Balances in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """The maximum number of Balances to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PricingListParams"]


class PricingListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    date: str
    """Date on which to retrieve active Pricings."""

    ids: List[str]
    """List of Pricing IDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi-page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Pricings to retrieve per page."""

    plan_id: Annotated[str, PropertyInfo(alias="planId")]
    """UUID of the Plan to retrieve Pricings for."""

    plan_template_id: Annotated[str, PropertyInfo(alias="planTemplateId")]
    """UUID of the PlanTemplate to retrieve Pricings for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .bills import (
    BillsResource,
    AsyncBillsResource,
    BillsResourceWithRawResponse,
    AsyncBillsResourceWithRawResponse,
    BillsResourceWithStreamingResponse,
    AsyncBillsResourceWithStreamingResponse,
)
from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .meters import (
    MetersResource,
    AsyncMetersResource,
    MetersResourceWithRawResponse,
    AsyncMetersResourceWithRawResponse,
    MetersResourceWithStreamingResponse,
    AsyncMetersResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .balances import (
    BalancesResource,
    AsyncBalancesResource,
    BalancesResourceWithRawResponse,
    AsyncBalancesResourceWithRawResponse,
    BalancesResourceWithStreamingResponse,
    AsyncBalancesResourceWithStreamingResponse,
)
from .counters import (
    CountersResource,
    AsyncCountersResource,
    CountersResourceWithRawResponse,
    AsyncCountersResourceWithRawResponse,
    CountersResourceWithStreamingResponse,
    AsyncCountersResourceWithStreamingResponse,
)
from .pricings import (
    PricingsResource,
    AsyncPricingsResource,
    PricingsResourceWithRawResponse,
    AsyncPricingsResourceWithRawResponse,
    PricingsResourceWithStreamingResponse,
    AsyncPricingsResourceWithStreamingResponse,
)
from .products import (
    ProductsResource,
    AsyncProductsResource,
    ProductsResourceWithRawResponse,
    AsyncProductsResourceWithRawResponse,
    ProductsResourceWithStreamingResponse,
    AsyncProductsResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from .bill_jobs import (
    BillJobsResource,
    AsyncBillJobsResource,
    BillJobsResourceWithRawResponse,
    AsyncBillJobsResourceWithRawResponse,
    BillJobsResourceWithStreamingResponse,
    AsyncBillJobsResourceWithStreamingResponse,
)
from .contracts import (
    ContractsResource,
    AsyncContractsResource,
    ContractsResourceWithRawResponse,
    AsyncContractsResourceWithRawResponse,
    ContractsResourceWithStreamingResponse,
    AsyncContractsResourceWithStreamingResponse,
)
from .currencies import (
    CurrenciesResource,
    AsyncCurrenciesResource,
    CurrenciesResourceWithRawResponse,
    AsyncCurrenciesResourceWithRawResponse,
    CurrenciesResourceWithStreamingResponse,
    AsyncCurrenciesResourceWithStreamingResponse,
)
from .bill_config import (
    BillConfigResource,
    AsyncBillConfigResource,
    BillConfigResourceWithRawResponse,
    AsyncBillConfigResourceWithRawResponse,
    BillConfigResourceWithStreamingResponse,
    AsyncBillConfigResourceWithStreamingResponse,
)
from .commitments import (
    CommitmentsResource,
    AsyncCommitmentsResource,
    CommitmentsResourceWithRawResponse,
    AsyncCommitmentsResourceWithRawResponse,
    CommitmentsResourceWithStreamingResponse,
    AsyncCommitmentsResourceWithStreamingResponse,
)
from .plan_groups import (
    PlanGroupsResource,
    AsyncPlanGroupsResource,
    PlanGroupsResourceWithRawResponse,
    AsyncPlanGroupsResourceWithRawResponse,
    PlanGroupsResourceWithStreamingResponse,
    AsyncPlanGroupsResourceWithStreamingResponse,
)
from .aggregations import (
    AggregationsResource,
    AsyncAggregationsResource,
    AggregationsResourceWithRawResponse,
    AsyncAggregationsResourceWithRawResponse,
    AggregationsResourceWithStreamingResponse,
    AsyncAggregationsResourceWithStreamingResponse,
)
from .data_exports import (
    DataExportsResource,
    AsyncDataExportsResource,
    DataExportsResourceWithRawResponse,
    AsyncDataExportsResourceWithRawResponse,
    DataExportsResourceWithStreamingResponse,
    AsyncDataExportsResourceWithStreamingResponse,
)
from .account_plans import (
    AccountPlansResource,
    AsyncAccountPlansResource,
    AccountPlansResourceWithRawResponse,
    AsyncAccountPlansResourceWithRawResponse,
    AccountPlansResourceWithStreamingResponse,
    AsyncAccountPlansResourceWithStreamingResponse,
)
from .custom_fields import (
    CustomFieldsResource,
    AsyncCustomFieldsResource,
    CustomFieldsResourceWithRawResponse,
    AsyncCustomFieldsResourceWithRawResponse,
    CustomFieldsResourceWithStreamingResponse,
    AsyncCustomFieldsResourceWithStreamingResponse,
)
from .debit_reasons import (
    DebitReasonsResource,
    AsyncDebitReasonsResource,
    DebitReasonsResourceWithRawResponse,
    AsyncDebitReasonsResourceWithRawResponse,
    DebitReasonsResourceWithStreamingResponse,
    AsyncDebitReasonsResourceWithStreamingResponse,
)
from .authentication import (
    AuthenticationResource,
    AsyncAuthenticationResource,
    AuthenticationResourceWithRawResponse,
    AsyncAuthenticationResourceWithRawResponse,
    AuthenticationResourceWithStreamingResponse,
    AsyncAuthenticationResourceWithStreamingResponse,
)
from .credit_reasons import (
    CreditReasonsResource,
    AsyncCreditReasonsResource,
    CreditReasonsResourceWithRawResponse,
    AsyncCreditReasonsResourceWithRawResponse,
    CreditReasonsResourceWithStreamingResponse,
    AsyncCreditReasonsResourceWithStreamingResponse,
)
from .plan_templates import (
    PlanTemplatesResource,
    AsyncPlanTemplatesResource,
    PlanTemplatesResourceWithRawResponse,
    AsyncPlanTemplatesResourceWithRawResponse,
    PlanTemplatesResourceWithStreamingResponse,
    AsyncPlanTemplatesResourceWithStreamingResponse,
)
from .resource_groups import (
    ResourceGroupsResource,
    AsyncResourceGroupsResource,
    ResourceGroupsResourceWithRawResponse,
    AsyncResourceGroupsResourceWithRawResponse,
    ResourceGroupsResourceWithStreamingResponse,
    AsyncResourceGroupsResourceWithStreamingResponse,
)
from .counter_pricings import (
    CounterPricingsResource,
    AsyncCounterPricingsResource,
    CounterPricingsResourceWithRawResponse,
    AsyncCounterPricingsResourceWithRawResponse,
    CounterPricingsResourceWithStreamingResponse,
    AsyncCounterPricingsResourceWithStreamingResponse,
)
from .plan_group_links import (
    PlanGroupLinksResource,
    AsyncPlanGroupLinksResource,
    PlanGroupLinksResourceWithRawResponse,
    AsyncPlanGroupLinksResourceWithRawResponse,
    PlanGroupLinksResourceWithStreamingResponse,
    AsyncPlanGroupLinksResourceWithStreamingResponse,
)
from .external_mappings import (
    ExternalMappingsResource,
    AsyncExternalMappingsResource,
    ExternalMappingsResourceWithRawResponse,
    AsyncExternalMappingsResourceWithRawResponse,
    ExternalMappingsResourceWithStreamingResponse,
    AsyncExternalMappingsResourceWithStreamingResponse,
)
from .transaction_types import (
    TransactionTypesResource,
    AsyncTransactionTypesResource,
    TransactionTypesResourceWithRawResponse,
    AsyncTransactionTypesResourceWithRawResponse,
    TransactionTypesResourceWithStreamingResponse,
    AsyncTransactionTypesResourceWithStreamingResponse,
)
from .counter_adjustments import (
    CounterAdjustmentsResource,
    AsyncCounterAdjustmentsResource,
    CounterAdjustmentsResourceWithRawResponse,
    AsyncCounterAdjustmentsResourceWithRawResponse,
    CounterAdjustmentsResourceWithStreamingResponse,
    AsyncCounterAdjustmentsResourceWithStreamingResponse,
)
from .organization_config import (
    OrganizationConfigResource,
    AsyncOrganizationConfigResource,
    OrganizationConfigResourceWithRawResponse,
    AsyncOrganizationConfigResourceWithRawResponse,
    OrganizationConfigResourceWithStreamingResponse,
    AsyncOrganizationConfigResourceWithStreamingResponse,
)
from .permission_policies import (
    PermissionPoliciesResource,
    AsyncPermissionPoliciesResource,
    PermissionPoliciesResourceWithRawResponse,
    AsyncPermissionPoliciesResourceWithRawResponse,
    PermissionPoliciesResourceWithStreamingResponse,
    AsyncPermissionPoliciesResourceWithStreamingResponse,
)
from .compound_aggregations import (
    CompoundAggregationsResource,
    AsyncCompoundAggregationsResource,
    CompoundAggregationsResourceWithRawResponse,
    AsyncCompoundAggregationsResourceWithRawResponse,
    CompoundAggregationsResourceWithStreamingResponse,
    AsyncCompoundAggregationsResourceWithStreamingResponse,
)
from .integration_configurations import (
    IntegrationConfigurationsResource,
    AsyncIntegrationConfigurationsResource,
    IntegrationConfigurationsResourceWithRawResponse,
    AsyncIntegrationConfigurationsResourceWithRawResponse,
    IntegrationConfigurationsResourceWithStreamingResponse,
    AsyncIntegrationConfigurationsResourceWithStreamingResponse,
)
from .notification_configurations import (
    NotificationConfigurationsResource,
    AsyncNotificationConfigurationsResource,
    NotificationConfigurationsResourceWithRawResponse,
    AsyncNotificationConfigurationsResourceWithRawResponse,
    NotificationConfigurationsResourceWithStreamingResponse,
    AsyncNotificationConfigurationsResourceWithStreamingResponse,
)
from .scheduled_event_configurations import (
    ScheduledEventConfigurationsResource,
    AsyncScheduledEventConfigurationsResource,
    ScheduledEventConfigurationsResourceWithRawResponse,
    AsyncScheduledEventConfigurationsResourceWithRawResponse,
    ScheduledEventConfigurationsResourceWithStreamingResponse,
    AsyncScheduledEventConfigurationsResourceWithStreamingResponse,
)

__all__ = [
    "AuthenticationResource",
    "AsyncAuthenticationResource",
    "AuthenticationResourceWithRawResponse",
    "AsyncAuthenticationResourceWithRawResponse",
    "AuthenticationResourceWithStreamingResponse",
    "AsyncAuthenticationResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "AccountPlansResource",
    "AsyncAccountPlansResource",
    "AccountPlansResourceWithRawResponse",
    "AsyncAccountPlansResourceWithRawResponse",
    "AccountPlansResourceWithStreamingResponse",
    "AsyncAccountPlansResourceWithStreamingResponse",
    "AggregationsResource",
    "AsyncAggregationsResource",
    "AggregationsResourceWithRawResponse",
    "AsyncAggregationsResourceWithRawResponse",
    "AggregationsResourceWithStreamingResponse",
    "AsyncAggregationsResourceWithStreamingResponse",
    "BalancesResource",
    "AsyncBalancesResource",
    "BalancesResourceWithRawResponse",
    "AsyncBalancesResourceWithRawResponse",
    "BalancesResourceWithStreamingResponse",
    "AsyncBalancesResourceWithStreamingResponse",
    "BillsResource",
    "AsyncBillsResource",
    "BillsResourceWithRawResponse",
    "AsyncBillsResourceWithRawResponse",
    "BillsResourceWithStreamingResponse",
    "AsyncBillsResourceWithStreamingResponse",
    "BillConfigResource",
    "AsyncBillConfigResource",
    "BillConfigResourceWithRawResponse",
    "AsyncBillConfigResourceWithRawResponse",
    "BillConfigResourceWithStreamingResponse",
    "AsyncBillConfigResourceWithStreamingResponse",
    "CommitmentsResource",
    "AsyncCommitmentsResource",
    "CommitmentsResourceWithRawResponse",
    "AsyncCommitmentsResourceWithRawResponse",
    "CommitmentsResourceWithStreamingResponse",
    "AsyncCommitmentsResourceWithStreamingResponse",
    "BillJobsResource",
    "AsyncBillJobsResource",
    "BillJobsResourceWithRawResponse",
    "AsyncBillJobsResourceWithRawResponse",
    "BillJobsResourceWithStreamingResponse",
    "AsyncBillJobsResourceWithStreamingResponse",
    "CompoundAggregationsResource",
    "AsyncCompoundAggregationsResource",
    "CompoundAggregationsResourceWithRawResponse",
    "AsyncCompoundAggregationsResourceWithRawResponse",
    "CompoundAggregationsResourceWithStreamingResponse",
    "AsyncCompoundAggregationsResourceWithStreamingResponse",
    "ContractsResource",
    "AsyncContractsResource",
    "ContractsResourceWithRawResponse",
    "AsyncContractsResourceWithRawResponse",
    "ContractsResourceWithStreamingResponse",
    "AsyncContractsResourceWithStreamingResponse",
    "CountersResource",
    "AsyncCountersResource",
    "CountersResourceWithRawResponse",
    "AsyncCountersResourceWithRawResponse",
    "CountersResourceWithStreamingResponse",
    "AsyncCountersResourceWithStreamingResponse",
    "CounterAdjustmentsResource",
    "AsyncCounterAdjustmentsResource",
    "CounterAdjustmentsResourceWithRawResponse",
    "AsyncCounterAdjustmentsResourceWithRawResponse",
    "CounterAdjustmentsResourceWithStreamingResponse",
    "AsyncCounterAdjustmentsResourceWithStreamingResponse",
    "CounterPricingsResource",
    "AsyncCounterPricingsResource",
    "CounterPricingsResourceWithRawResponse",
    "AsyncCounterPricingsResourceWithRawResponse",
    "CounterPricingsResourceWithStreamingResponse",
    "AsyncCounterPricingsResourceWithStreamingResponse",
    "CreditReasonsResource",
    "AsyncCreditReasonsResource",
    "CreditReasonsResourceWithRawResponse",
    "AsyncCreditReasonsResourceWithRawResponse",
    "CreditReasonsResourceWithStreamingResponse",
    "AsyncCreditReasonsResourceWithStreamingResponse",
    "CurrenciesResource",
    "AsyncCurrenciesResource",
    "CurrenciesResourceWithRawResponse",
    "AsyncCurrenciesResourceWithRawResponse",
    "CurrenciesResourceWithStreamingResponse",
    "AsyncCurrenciesResourceWithStreamingResponse",
    "CustomFieldsResource",
    "AsyncCustomFieldsResource",
    "CustomFieldsResourceWithRawResponse",
    "AsyncCustomFieldsResourceWithRawResponse",
    "CustomFieldsResourceWithStreamingResponse",
    "AsyncCustomFieldsResourceWithStreamingResponse",
    "DataExportsResource",
    "AsyncDataExportsResource",
    "DataExportsResourceWithRawResponse",
    "AsyncDataExportsResourceWithRawResponse",
    "DataExportsResourceWithStreamingResponse",
    "AsyncDataExportsResourceWithStreamingResponse",
    "DebitReasonsResource",
    "AsyncDebitReasonsResource",
    "DebitReasonsResourceWithRawResponse",
    "AsyncDebitReasonsResourceWithRawResponse",
    "DebitReasonsResourceWithStreamingResponse",
    "AsyncDebitReasonsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "ExternalMappingsResource",
    "AsyncExternalMappingsResource",
    "ExternalMappingsResourceWithRawResponse",
    "AsyncExternalMappingsResourceWithRawResponse",
    "ExternalMappingsResourceWithStreamingResponse",
    "AsyncExternalMappingsResourceWithStreamingResponse",
    "IntegrationConfigurationsResource",
    "AsyncIntegrationConfigurationsResource",
    "IntegrationConfigurationsResourceWithRawResponse",
    "AsyncIntegrationConfigurationsResourceWithRawResponse",
    "IntegrationConfigurationsResourceWithStreamingResponse",
    "AsyncIntegrationConfigurationsResourceWithStreamingResponse",
    "MetersResource",
    "AsyncMetersResource",
    "MetersResourceWithRawResponse",
    "AsyncMetersResourceWithRawResponse",
    "MetersResourceWithStreamingResponse",
    "AsyncMetersResourceWithStreamingResponse",
    "NotificationConfigurationsResource",
    "AsyncNotificationConfigurationsResource",
    "NotificationConfigurationsResourceWithRawResponse",
    "AsyncNotificationConfigurationsResourceWithRawResponse",
    "NotificationConfigurationsResourceWithStreamingResponse",
    "AsyncNotificationConfigurationsResourceWithStreamingResponse",
    "OrganizationConfigResource",
    "AsyncOrganizationConfigResource",
    "OrganizationConfigResourceWithRawResponse",
    "AsyncOrganizationConfigResourceWithRawResponse",
    "OrganizationConfigResourceWithStreamingResponse",
    "AsyncOrganizationConfigResourceWithStreamingResponse",
    "PermissionPoliciesResource",
    "AsyncPermissionPoliciesResource",
    "PermissionPoliciesResourceWithRawResponse",
    "AsyncPermissionPoliciesResourceWithRawResponse",
    "PermissionPoliciesResourceWithStreamingResponse",
    "AsyncPermissionPoliciesResourceWithStreamingResponse",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
    "PlanGroupsResource",
    "AsyncPlanGroupsResource",
    "PlanGroupsResourceWithRawResponse",
    "AsyncPlanGroupsResourceWithRawResponse",
    "PlanGroupsResourceWithStreamingResponse",
    "AsyncPlanGroupsResourceWithStreamingResponse",
    "PlanGroupLinksResource",
    "AsyncPlanGroupLinksResource",
    "PlanGroupLinksResourceWithRawResponse",
    "AsyncPlanGroupLinksResourceWithRawResponse",
    "PlanGroupLinksResourceWithStreamingResponse",
    "AsyncPlanGroupLinksResourceWithStreamingResponse",
    "PlanTemplatesResource",
    "AsyncPlanTemplatesResource",
    "PlanTemplatesResourceWithRawResponse",
    "AsyncPlanTemplatesResourceWithRawResponse",
    "PlanTemplatesResourceWithStreamingResponse",
    "AsyncPlanTemplatesResourceWithStreamingResponse",
    "PricingsResource",
    "AsyncPricingsResource",
    "PricingsResourceWithRawResponse",
    "AsyncPricingsResourceWithRawResponse",
    "PricingsResourceWithStreamingResponse",
    "AsyncPricingsResourceWithStreamingResponse",
    "ProductsResource",
    "AsyncProductsResource",
    "ProductsResourceWithRawResponse",
    "AsyncProductsResourceWithRawResponse",
    "ProductsResourceWithStreamingResponse",
    "AsyncProductsResourceWithStreamingResponse",
    "ResourceGroupsResource",
    "AsyncResourceGroupsResource",
    "ResourceGroupsResourceWithRawResponse",
    "AsyncResourceGroupsResourceWithRawResponse",
    "ResourceGroupsResourceWithStreamingResponse",
    "AsyncResourceGroupsResourceWithStreamingResponse",
    "ScheduledEventConfigurationsResource",
    "AsyncScheduledEventConfigurationsResource",
    "ScheduledEventConfigurationsResourceWithRawResponse",
    "AsyncScheduledEventConfigurationsResourceWithRawResponse",
    "ScheduledEventConfigurationsResourceWithStreamingResponse",
    "AsyncScheduledEventConfigurationsResourceWithStreamingResponse",
    "TransactionTypesResource",
    "AsyncTransactionTypesResource",
    "TransactionTypesResourceWithRawResponse",
    "AsyncTransactionTypesResourceWithRawResponse",
    "TransactionTypesResourceWithStreamingResponse",
    "AsyncTransactionTypesResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "WebhooksResource",
    "AsyncWebhooksResource",
    "WebhooksResourceWithRawResponse",
    "AsyncWebhooksResourceWithRawResponse",
    "WebhooksResourceWithStreamingResponse",
    "AsyncWebhooksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .commitment_fee_param import CommitmentFeeParam

__all__ = ["CommitmentUpdateParams"]


class CommitmentUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """
    The unique identifier (UUID) for the end customer Account the Commitment is
    added to.
    """

    amount: Required[float]
    """The total amount that the customer has committed to pay."""

    currency: Required[str]
    """The currency used for the Commitment. For example: USD."""

    end_date: Required[Annotated[Union[str, date], PropertyInfo(alias="endDate", format="iso8601")]]
    """The end date of the Commitment period in ISO-8601 format.

    **Note:** End date is exclusive - if you set an end date of June 1st 2022, then
    the Commitment ceases to be active for the Account at midnight on May 31st 2022,
    and any Prepayment fees due are calculated up to that point in time, NOT up to
    midnight on June 1st
    """

    start_date: Required[Annotated[Union[str, date], PropertyInfo(alias="startDate", format="iso8601")]]
    """The start date of the Commitment period in ISO-8601 format."""

    accounting_product_id: Annotated[str, PropertyInfo(alias="accountingProductId")]
    """
    The unique identifier (UUID) for the Product linked to the Commitment for
    accounting purposes. _(Optional)_
    """

    amount_first_bill: Annotated[float, PropertyInfo(alias="amountFirstBill")]
    """The amount to be billed in the first invoice."""

    amount_pre_paid: Annotated[float, PropertyInfo(alias="amountPrePaid")]
    """
    The amount that the customer has already paid upfront at the start of the
    Commitment service period.
    """

    bill_epoch: Annotated[Union[str, date], PropertyInfo(alias="billEpoch", format="iso8601")]
    """
    The starting date _(in ISO-8601 date format)_ from which the billing cycles are
    calculated.
    """

    billing_interval: Annotated[int, PropertyInfo(alias="billingInterval")]
    """How often the Commitment fees are applied to bills.

    For example, if the plan being used to bill for Commitment fees is set to issue
    bills every three months and the `billingInterval` is set to 2, then the
    Commitment fees are applied every six months.
    """

    billing_offset: Annotated[int, PropertyInfo(alias="billingOffset")]
    """
    Defines an offset for when the Commitment fees are first applied to bills on the
    Account. For example, if bills are issued every three months and the
    `billingOffset` is 0, then the charge is applied to the first bill (at three
    months); if set to 1, it's applied to the next bill (at six months), and so on.
    """

    billing_plan_id: Annotated[str, PropertyInfo(alias="billingPlanId")]
    """
    The unique identifier (UUID) for the Product Plan used for billing Commitment
    fees due.
    """

    child_billing_mode: Annotated[
        Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"], PropertyInfo(alias="childBillingMode")
    ]
    """
    If the Account is either a Parent or a Child Account, this specifies the Account
    hierarchy billing mode. The mode determines how billing will be handled and
    shown on bills for charges due on the Parent Account, and charges due on Child
    Accounts:

    - **Parent Breakdown** - a separate bill line item per Account. Default setting.

    - **Parent Summary** - single bill line item for all Accounts.

    - **Child** - the Child Account is billed.
    """

    commitment_fee_bill_in_advance: Annotated[bool, PropertyInfo(alias="commitmentFeeBillInAdvance")]
    """
    A boolean value indicating whether the Commitment fee is billed in advance
    _(start of each billing period)_ or arrears _(end of each billing period)_.

    If no value is supplied, then the Organization Configuration value is used.

    - **TRUE** - bill in advance _(start of each billing period)_.
    - **FALSE** - bill in arrears _(end of each billing period)_.
    """

    commitment_fee_description: Annotated[str, PropertyInfo(alias="commitmentFeeDescription")]
    """A textual description of the Commitment fee."""

    commitment_usage_description: Annotated[str, PropertyInfo(alias="commitmentUsageDescription")]
    """A textual description of the Commitment usage."""

    contract_id: Annotated[str, PropertyInfo(alias="contractId")]
    """
    The unique identifier (UUID) for a Contract you've created for the Account -
    used to add the Commitment to this Contract.

    **Note:** If you associate the Commitment with a Contract you must ensure the
    Account Plan attached to the Account has the same Contract associated with it.
    If the Account Plan Contract and Commitment Contract do not match, then at
    billing the Commitment amount will not be drawn-down against.
    """

    drawdowns_accounting_product_id: Annotated[str, PropertyInfo(alias="drawdownsAccountingProductId")]
    """
    Optional Product ID this Commitment consumptions should be attributed to for
    accounting purposes
    """

    fee_dates: Annotated[Iterable[CommitmentFeeParam], PropertyInfo(alias="feeDates")]
    """Used for billing any outstanding Commitment fees _on a schedule_.

    Create an array to define a series of bill dates and amounts covering specified
    service periods:

    - `date` - the billing date _(in ISO-8601 format)_.
    - `amount` - the billed amount.
    - `servicePeriodStartDate` and `servicePeriodEndDate` - defines the service
      period the bill covers _(in ISO-8601 format)_.

    **Notes:**

    - If you try to set `servicePeriodStartDate` _after_ `servicePeriodEndDate`,
      you'll receive an error.
    - You can set `servicePeriodStartDate` and `servicePeriodEndDate` to the _same
      date_ without receiving an error, but _please be sure_ your Commitment billing
      use case requires this.
    """

    fees_accounting_product_id: Annotated[str, PropertyInfo(alias="feesAccountingProductId")]
    """
    Optional Product ID this Commitment fees should be attributed to for accounting
    purposes
    """

    line_item_types: Annotated[
        List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ],
        PropertyInfo(alias="lineItemTypes"),
    ]
    """
    Specify the line item charge types that can draw-down at billing against the
    Commitment amount. Options are:

    - `MINIMUM_SPEND`
    - `STANDING_CHARGE`
    - `USAGE`
    - `"COUNTER_RUNNING_TOTAL_CHARGE"`
    - `"COUNTER_ADJUSTMENT_DEBIT"`

    **NOTE:** If no charge types are specified, by default _all types_ can draw-down
    against the Commitment amount at billing.
    """

    overage_description: Annotated[str, PropertyInfo(alias="overageDescription")]
    """A textual description of the overage charges."""

    overage_surcharge_percent: Annotated[float, PropertyInfo(alias="overageSurchargePercent")]
    """
    The percentage surcharge applied to usage charges that exceed the Commitment
    amount.

    **Note:** You can enter a _negative percentage_ if you want to give a discount
    rate for usage to end customers who exceed their Commitment amount
    """

    product_ids: Annotated[List[str], PropertyInfo(alias="productIds")]
    """A list of unique identifiers (UUIDs) for Products the Account consumes.

    Charges due for these Products will be made available for draw-down against the
    Commitment.

    **Note:** If not used, then charges due for all Products the Account consumes
    will be made available for draw-down against the Commitment.
    """

    separate_overage_usage: Annotated[bool, PropertyInfo(alias="separateOverageUsage")]
    """
    A boolean value indicating whether the overage usage is billed separately or
    together. If overage usage is separated and a Commitment amount has been
    consumed by an Account, any subsequent line items on Bills against the Account
    for usage will show as separate "overage usage" charges, not simply as "usage"
    charges:

    - **TRUE** - billed separately.
    - **FALSE** - billed together.

    **Notes:**

    - Can be used only if no value or 0 has been defined for the
      `overageSurchargePercent` parameter. If you try to separate overage usage when
      a value other than 0 has been defined for `overageSurchargePercent`, you'll
      receive an error.
    - If a priced Plan is used to bill any outstanding Commitment fees due and the
      Plan is set up with overage pricing on a _tiered pricing structure_ and you
      enable separate bill line items for overage usage, then overage usage charges
      will be rated according to the overage pricing defined for the tiered pricing
      on the Plan.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AccountPlanResponse"]


class AccountPlanResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """
    The unique identifier (UUID) for the Account to which the AccountPlan or
    AccounPlanGroup is attached.
    """

    bill_epoch: Optional[date] = FieldInfo(alias="billEpoch", default=None)
    """
    The initial date for creating the first bill against the Account for charges due
    under the AccountPlan or AccountPlanGroup. All subsequent bill creation dates
    are calculated from this date. If left empty, the first bill date definedfor the
    Account is used. The date is in ISO-8601 format.
    """

    child_billing_mode: Optional[Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"]] = FieldInfo(
        alias="childBillingMode", default=None
    )
    """
    If the Account is either a Parent or a Child Account, this specifies the Account
    hierarchy billing mode. The mode determines how billing will be handled and
    shown on bills for charges due on the Parent Account, and charges due on Child
    Accounts:

    - **Parent Breakdown** - a separate bill line item per Account. Default setting.

    - **Parent Summary** - single bill line item for all Accounts.

    - **Child** - the Child Account is billed.
    """

    code: Optional[str] = None
    """The unique short code of the AccountPlan or AccountPlanGroup."""

    contract_id: Optional[str] = FieldInfo(alias="contractId", default=None)
    """
    The unique identifier (UUID) for the Contract to which the Plan or Plan Group
    attached to the Account has been added.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """
    The unique identifier (UUID) for the user who created the AccountPlan or
    AccountPlanGroup.
    """

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time _(in ISO 8601 format)_ when the AccountPlan or
    AccountPlanGroup was first created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time _(in ISO 8601 format)_ when the AccountPlan or
    AccountPlanGroup was last modified.
    """

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """
    The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
    ceases to be active for the Account. If not specified, the AccountPlan or
    AccountPlanGroup remains active indefinitely.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) for the user who last modified the AccountPlan or
    AccountPlanGroup.
    """

    plan_group_id: Optional[str] = FieldInfo(alias="planGroupId", default=None)
    """
    The unique identifier (UUID) of the Plan Group that has been attached to the
    Account to create the AccountPlanGroup.
    """

    plan_id: Optional[str] = FieldInfo(alias="planId", default=None)
    """
    The unique identifier (UUID) of the Plan that has been attached to the Account
    to create the AccountPlan.
    """

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """The unique identifier (UUID) for the Product associated with the AccountPlan.

    **Note:** Not present in response for AccountPlanGroup - Plan Groups can contain
    multiple Plans belonging to different Products.
    """

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """
    The start date _(in ISO-8601 format)_ for the when the AccountPlan or
    AccountPlanGroup starts to be active for the Account.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime

import httpx

from ...types import usage_query_params, usage_submit_params, usage_get_failed_ingest_download_url_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .file_uploads.file_uploads import (
    FileUploadsResource,
    AsyncFileUploadsResource,
    FileUploadsResourceWithRawResponse,
    AsyncFileUploadsResourceWithRawResponse,
    FileUploadsResourceWithStreamingResponse,
    AsyncFileUploadsResourceWithStreamingResponse,
)
from ...types.usage_query_response import UsageQueryResponse
from ...types.download_url_response import DownloadURLResponse
from ...types.submit_measurements_response import SubmitMeasurementsResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def file_uploads(self) -> FileUploadsResource:
        return FileUploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def get_failed_ingest_download_url(
        self,
        *,
        org_id: str | None = None,
        file: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DownloadURLResponse:
        """
        Returns a presigned download URL for failed ingest file download based on the
        file path provided.

        If a usage data ingest measurement you submit to the m3ter platform fails, an
        `ingest.validation.failure` Event is generated. Use this call to obtain a
        download URL which you can then use to download a file containing details of
        what went wrong with the attempted usage data measurement ingest, and allowing
        you to follow-up and resolve the issue.

        To obtain the `file` query parameter:

        - Use the
          [List Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventFields)
          call with the `ingest.validation.failure` for the `eventName` query parameter.
        - The response contains a `getDownloadUrl` response parameter and this contains
          the file path you can use to obtain the failed ingest file download URL.

        **Notes:**

        - The presigned Url returned to use for failed ingest file download is
          time-bound and expires after 5 minutes.
        - If you make a List Events call for `ingest.validation.failure` Events in your
          Organization, then you can perform this **GET** call using the full URL
          returned for any ingest failure Event to obtain a failed ingest file download
          URL for the Event.

        Args:
          file: The file path

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/measurements/failedIngest/getDownloadUrl",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"file": file}, usage_get_failed_ingest_download_url_params.UsageGetFailedIngestDownloadURLParams
                ),
            ),
            cast_to=DownloadURLResponse,
        )

    def query(
        self,
        *,
        org_id: str | None = None,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregations: Iterable[usage_query_params.Aggregation] | NotGiven = NOT_GIVEN,
        dimension_filters: Iterable[usage_query_params.DimensionFilter] | NotGiven = NOT_GIVEN,
        groups: Iterable[usage_query_params.Group] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UsageQueryResponse:
        """
        Query and filter usage data

        Args:
          end_date: ISO 8601 formatted end date to filter by.

          start_date: ISO 8601 formatted start date to filter by.

          account_ids

          aggregations

          dimension_filters

          groups

          limit

          meter_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/usage/query",
            body=maybe_transform(
                {
                    "end_date": end_date,
                    "start_date": start_date,
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "groups": groups,
                    "limit": limit,
                    "meter_ids": meter_ids,
                },
                usage_query_params.UsageQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageQueryResponse,
        )

    def submit(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[usage_submit_params.Measurement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubmitMeasurementsResponse:
        """Submit a measurement or multiple measurements to the m3ter platform.

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submittting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submittting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/measurements",
            body=maybe_transform({"measurements": measurements}, usage_submit_params.UsageSubmitParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubmitMeasurementsResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResource:
        return AsyncFileUploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def get_failed_ingest_download_url(
        self,
        *,
        org_id: str | None = None,
        file: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DownloadURLResponse:
        """
        Returns a presigned download URL for failed ingest file download based on the
        file path provided.

        If a usage data ingest measurement you submit to the m3ter platform fails, an
        `ingest.validation.failure` Event is generated. Use this call to obtain a
        download URL which you can then use to download a file containing details of
        what went wrong with the attempted usage data measurement ingest, and allowing
        you to follow-up and resolve the issue.

        To obtain the `file` query parameter:

        - Use the
          [List Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventFields)
          call with the `ingest.validation.failure` for the `eventName` query parameter.
        - The response contains a `getDownloadUrl` response parameter and this contains
          the file path you can use to obtain the failed ingest file download URL.

        **Notes:**

        - The presigned Url returned to use for failed ingest file download is
          time-bound and expires after 5 minutes.
        - If you make a List Events call for `ingest.validation.failure` Events in your
          Organization, then you can perform this **GET** call using the full URL
          returned for any ingest failure Event to obtain a failed ingest file download
          URL for the Event.

        Args:
          file: The file path

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/measurements/failedIngest/getDownloadUrl",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"file": file}, usage_get_failed_ingest_download_url_params.UsageGetFailedIngestDownloadURLParams
                ),
            ),
            cast_to=DownloadURLResponse,
        )

    async def query(
        self,
        *,
        org_id: str | None = None,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        aggregations: Iterable[usage_query_params.Aggregation] | NotGiven = NOT_GIVEN,
        dimension_filters: Iterable[usage_query_params.DimensionFilter] | NotGiven = NOT_GIVEN,
        groups: Iterable[usage_query_params.Group] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        meter_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UsageQueryResponse:
        """
        Query and filter usage data

        Args:
          end_date: ISO 8601 formatted end date to filter by.

          start_date: ISO 8601 formatted start date to filter by.

          account_ids

          aggregations

          dimension_filters

          groups

          limit

          meter_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/usage/query",
            body=await async_maybe_transform(
                {
                    "end_date": end_date,
                    "start_date": start_date,
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "groups": groups,
                    "limit": limit,
                    "meter_ids": meter_ids,
                },
                usage_query_params.UsageQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageQueryResponse,
        )

    async def submit(
        self,
        *,
        org_id: str | None = None,
        measurements: Iterable[usage_submit_params.Measurement],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubmitMeasurementsResponse:
        """Submit a measurement or multiple measurements to the m3ter platform.

        The maximum
        size of the payload needs to be less than 512,000 bytes.

        **NOTES:**

        - **Non-existent Accounts.** The `account` request parameter is required.
          However, if you want to submit a usage data measurement for an Account which
          does not yet exist in your Organization, you can use an `account` code for a
          non-existent Account. A new skeleton Account will be automatically created.
          The usage data measurement is accepted and ingested as data belonging to the
          new auto-created Account. At a later date, you can edit the Account's
          Code,??Name, and??e-mail address. For more details, see
          [Submittting Usage Data for Non-Existent Accounts](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-for-non-existent-accounts)
          in our main documentation.
        - **Usage Data Adjustments.** If you need to make corrections for billing
          retrospectively against an Account, you can use date/time values in the past
          for the `ts` (timestamp) request parameter to submit positive or negative
          usage data amounts to correct and reconcile earlier billing anomalies. For
          more details, see
          [Submittting Usage Data Adjustments Using Timestamp](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/submitting-usage-data-adjustments-using-timestamp)
          in our main documentation.
        - **Ingest Validation Failure Events.** After the intial submission of a usage
          data measurement to the Ingest API, a data enrichment stage is performed to
          check for any errors in the usage data measurement, such as a missing field.
          If an error is identified, this might result in the submission being rejected.
          In these cases, an _ingest validation failure_ Event is generated, which you
          can review on the
          [Ingest Events](https://www.m3ter.com/docs/guides/billing-and-usage-data/submitting-usage-data/reviewing-and-resolving-ingest-events)
          page in the Console. See also the
          [Events](https://www.m3ter.com/docs/api#tag/Events) section in this API
          Reference.

        **IMPORTANT! - Use of PII:** The use of any of your end-customers' Personally
        Identifiable Information (PII) in m3ter is restricted to a few fields on the
        **Account** entity. Please ensure that any measurements you submit do not
        contain any end-customer PII data. See the
        [Introduction section](https://www.m3ter.com/docs/api#section/Introduction)
        above for more details.

        Args:
          measurements: Request containing the usage data measurements for submission.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/measurements",
            body=await async_maybe_transform({"measurements": measurements}, usage_submit_params.UsageSubmitParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubmitMeasurementsResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = to_raw_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = to_raw_response_wrapper(
            usage.query,
        )
        self.submit = to_raw_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> FileUploadsResourceWithRawResponse:
        return FileUploadsResourceWithRawResponse(self._usage.file_uploads)


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = async_to_raw_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = async_to_raw_response_wrapper(
            usage.query,
        )
        self.submit = async_to_raw_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResourceWithRawResponse:
        return AsyncFileUploadsResourceWithRawResponse(self._usage.file_uploads)


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = to_streamed_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = to_streamed_response_wrapper(
            usage.query,
        )
        self.submit = to_streamed_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> FileUploadsResourceWithStreamingResponse:
        return FileUploadsResourceWithStreamingResponse(self._usage.file_uploads)


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.get_failed_ingest_download_url = async_to_streamed_response_wrapper(
            usage.get_failed_ingest_download_url,
        )
        self.query = async_to_streamed_response_wrapper(
            usage.query,
        )
        self.submit = async_to_streamed_response_wrapper(
            usage.submit,
        )

    @cached_property
    def file_uploads(self) -> AsyncFileUploadsResourceWithStreamingResponse:
        return AsyncFileUploadsResourceWithStreamingResponse(self._usage.file_uploads)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccountEndDateBillingEntitiesParams"]


class AccountEndDateBillingEntitiesParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    billing_entities: Required[
        Annotated[
            List[Literal["CONTRACT", "ACCOUNTPLAN", "PREPAYMENT", "PRICINGS", "COUNTER_PRICINGS"]],
            PropertyInfo(alias="billingEntities"),
        ]
    ]
    """
    Defines which billing entities associated with the Account will have the
    specified end-date applied. For example, if you want the specified end-date to
    be applied to all Prepayments/Commitments created for the Account use
    `"PREPAYMENT"`.
    """

    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]]
    """
    The end date and time applied to the specified billing entities _(in ISO 8601
    format)_.
    """

    apply_to_children: Annotated[bool, PropertyInfo(alias="applyToChildren")]
    """A Boolean TRUE/FALSE flag.

    For Parent Accounts, set to TRUE if you want the specified end-date to be
    applied to any billing entities associated with Child Accounts. _(Optional)_
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CustomFieldsResponse"]


class CustomFieldsResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to Account entities."""

    account_plan: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="accountPlan", default=None)
    """CustomFields added to accountPlan entities."""

    aggregation: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to simple Aggregation entities."""

    compound_aggregation: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="compoundAggregation", default=None)
    """CustomFields added to Compound Aggregation entities."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this custom field."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the Organization was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The DateTime when a custom field was last modified - created, modified, or
    deleted - for the Organization _(in ISO-8601 format)_.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this custom field."""

    meter: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to Meter entities."""

    organization: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to the Organization."""

    plan: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to Plan entities."""

    plan_template: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="planTemplate", default=None)
    """CustomFields added to planTemplate entities."""

    product: Optional[Dict[str, Union[str, float]]] = None
    """CustomFields added to Product entities."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SubmitMeasurementsResponse"]


class SubmitMeasurementsResponse(BaseModel):
    result: Optional[str] = None
    """`accepted` is returned when successful."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ScheduleCreateParams", "OperationalDataExportScheduleRequest", "UsageDataExportScheduleRequest"]


class OperationalDataExportScheduleRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    operational_data_types: Required[
        Annotated[
            List[
                Literal[
                    "BILLS",
                    "COMMITMENTS",
                    "ACCOUNTS",
                    "BALANCES",
                    "CONTRACTS",
                    "ACCOUNT_PLANS",
                    "AGGREGATIONS",
                    "PLANS",
                    "PRICING",
                    "PRICING_BANDS",
                    "BILL_LINE_ITEMS",
                    "METERS",
                    "PRODUCTS",
                    "COMPOUND_AGGREGATIONS",
                    "PLAN_GROUPS",
                    "PLAN_GROUP_LINKS",
                    "PLAN_TEMPLATES",
                    "BALANCE_TRANSACTIONS",
                ]
            ],
            PropertyInfo(alias="operationalDataTypes"),
        ]
    ]
    """A list of the entities whose operational data is included in the data export."""

    source_type: Required[Annotated[Literal["USAGE", "OPERATIONAL"], PropertyInfo(alias="sourceType")]]

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


class UsageDataExportScheduleRequest(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    aggregation_frequency: Required[
        Annotated[Literal["ORIGINAL", "HOUR", "DAY", "WEEK", "MONTH"], PropertyInfo(alias="aggregationFrequency")]
    ]
    """
    Specifies the time period for the aggregation of usage data included each time
    the Data Export Schedule runs:

    - **ORIGINAL**. Usage data is _not aggregated_. If you select to not aggregate,
      then raw usage data measurements collected by all Data Field types and any
      Derived Fields on the selected Meters are included in the export. This is the
      _Default_.

    If you want to aggregate usage data for the Export Schedule you must define an
    `aggregationFrequency`:

    - **HOUR**. Aggregated hourly.
    - **DAY**. Aggregated daily.
    - **WEEK**. Aggregated weekly.
    - **MONTH**. Aggregated monthly.

    - If you select to aggregate usage data for a Export Schedule, then only the
      aggregated usage data collected by numeric Data Fields of type **MEASURE**,
      **INCOME**, or **COST** on selected Meters are included in the export.

    **NOTE**: If you define an `aggregationFrequency` other than **ORIGINAL** and do
    not define an `aggregation` method, then you'll receive and error.
    """

    source_type: Required[Annotated[Literal["USAGE", "OPERATIONAL"], PropertyInfo(alias="sourceType")]]

    time_period: Required[
        Annotated[
            Literal[
                "TODAY",
                "YESTERDAY",
                "WEEK_TO_DATE",
                "CURRENT_MONTH",
                "LAST_30_DAYS",
                "LAST_35_DAYS",
                "PREVIOUS_WEEK",
                "PREVIOUS_MONTH",
            ],
            PropertyInfo(alias="timePeriod"),
        ]
    ]
    """
    Define a time period to control the range of usage data you want the data export
    to contain when it runs:

    - **TODAY**. Data collected for the current day up until the time the export
      runs.
    - **YESTERDAY**. Data collected for the day before the export runs - that is,
      the 24 hour period from midnight to midnight of the day before.
    - **WEEK_TO_DATE**. Data collected for the period covering the current week to
      the date and time the export runs, and weeks run Monday to Monday.
    - **CURRENT_MONTH**. Data collected for the current month in which the export is
      ran up to and including the date and time the export runs.
    - **LAST_30_DAYS**. Data collected for the 30 days prior to the date the export
      is ran.
    - **LAST_35_DAYS**. Data collected for the 35 days prior to the date the export
      is ran.
    - **PREVIOUS_WEEK**. Data collected for the previous full week period, and weeks
      run Monday to Monday.
    - **PREVIOUS_MONTH**. Data collected for the previous full month period.

    For more details and examples, see the
    [Time Period](https://www.m3ter.com/docs/guides/data-exports/creating-export-schedules#time-period)
    section in our main User Documentation.
    """

    account_ids: Annotated[List[str], PropertyInfo(alias="accountIds")]
    """List of account IDs for which the usage data will be exported."""

    aggregation: Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN"]
    """
    Specifies the aggregation method applied to usage data collected in the numeric
    Data Fields of Meters included for the Data Export Schedule - that is, Data
    Fields of type **MEASURE**, **INCOME**, or **COST**:

    - **SUM**. Adds the values.
    - **MIN**. Uses the minimum value.
    - **MAX**. Uses the maximum value.
    - **COUNT**. Counts the number of values.
    - **LATEST**. Uses the most recent value. Note: Based on the timestamp `ts`
      value of usage data measurement submissions. If using this method, please
      ensure _distinct_ `ts` values are used for usage data measurement submissions.
    """

    meter_ids: Annotated[List[str], PropertyInfo(alias="meterIds")]
    """List of meter IDs for which the usage data will be exported."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """


ScheduleCreateParams: TypeAlias = Union[OperationalDataExportScheduleRequest, UsageDataExportScheduleRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.pricing_band import PricingBand

__all__ = ["CounterPricingCreateParams"]


class CounterPricingCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    counter_id: Required[Annotated[str, PropertyInfo(alias="counterId")]]
    """UUID of the Counter used to create the pricing."""

    pricing_bands: Required[Annotated[Iterable[PricingBand], PropertyInfo(alias="pricingBands")]]

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """
    The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
    for the Plan of Plan Template._(Required)_
    """

    accounting_product_id: Annotated[str, PropertyInfo(alias="accountingProductId")]
    """
    Optional Product ID this Pricing should be attributed to for accounting purposes
    """

    code: str
    """Unique short code for the Pricing."""

    cumulative: bool
    """
    Controls whether or not charge rates under a set of pricing bands configured for
    a Pricing are applied according to each separate band or at the highest band
    reached.

    _(Optional)_. The default value is **FALSE**.

    - When TRUE, at billing charge rates are applied according to each separate
      band.

    - When FALSE, at billing charge rates are applied according to highest band
      reached.

    **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
    require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
    set to **FALSE**.
    """

    description: str
    """Displayed on Bill line items."""

    end_date: Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]
    """
    The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
    the Plan or Plan Template.

    _(Optional)_ If not specified, the Pricing remains active indefinitely.
    """

    plan_id: Annotated[str, PropertyInfo(alias="planId")]
    """UUID of the Plan the Pricing is created for."""

    plan_template_id: Annotated[str, PropertyInfo(alias="planTemplateId")]
    """UUID of the Plan Template the Pricing is created for."""

    pro_rate_adjustment_credit: Annotated[bool, PropertyInfo(alias="proRateAdjustmentCredit")]
    """The default value is **TRUE**.

    - When TRUE, counter adjustment credits are prorated and are billed according to
      the number of days in billing period.

    - When FALSE, counter adjustment credits are not prorated and are billed for the
      entire billing period.

    _(Optional)_.
    """

    pro_rate_adjustment_debit: Annotated[bool, PropertyInfo(alias="proRateAdjustmentDebit")]
    """The default value is **TRUE**.

    - When TRUE, counter adjustment debits are prorated and are billed according to
      the number of days in billing period.

    - When FALSE, counter adjustment debits are not prorated and are billed for the
      entire billing period.

    _(Optional)_.
    """

    pro_rate_running_total: Annotated[bool, PropertyInfo(alias="proRateRunningTotal")]
    """The default value is **TRUE**.

    - When TRUE, counter running total charges are prorated and are billed according
      to the number of days in billing period.

    - When FALSE, counter running total charges are not prorated and are billed for
      the entire billing period.

    _(Optional)_.
    """

    running_total_bill_in_advance: Annotated[bool, PropertyInfo(alias="runningTotalBillInAdvance")]
    """The default value is **TRUE**.

    - When TRUE, running totals are billed at the start of each billing period.

    - When FALSE, running totals are billed at the end of each billing period.

    _(Optional)_.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["NotificationConfigurationCreateParams"]


class NotificationConfigurationCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    code: Required[str]
    """The short code for the Notification."""

    description: Required[str]
    """
    The description for the Notification providing a brief overview of its purpose
    and functionality.
    """

    event_name: Required[Annotated[str, PropertyInfo(alias="eventName")]]
    """The name of the _Event type_ that the Notification is based on.

    When an Event of this type occurs and any calculation built into the
    Notification evaluates to `True`, the Notification is triggered.

    **Note:** If the Notification is set to always fire, then the Notification will
    always be sent when the Event of the type it is based on occurs, and without any
    other conditions defined by a calculation having to be met.
    """

    name: Required[str]
    """The name of the Notification."""

    active: bool
    """Boolean flag that sets the Notification as active or inactive.

    Only active Notifications are sent when triggered by the Event they are based
    on:

    - **TRUE** - set Notification as active.
    - **FALSE** - set Notification as inactive.
    """

    always_fire_event: Annotated[bool, PropertyInfo(alias="alwaysFireEvent")]
    """
    A Boolean flag indicating whether the Notification is always triggered,
    regardless of other conditions and omitting reference to any calculation. This
    means the Notification will be triggered simply by the Event it is based on
    occurring and with no further conditions having to be met.

    - **TRUE** - the Notification is always triggered and omits any reference to the
      calculation to check for other conditions being true before triggering the
      Notification.
    - **FALSE** - the Notification is only triggered when the Event it is based on
      occurs and any calculation is checked and all conditions defined by the
      calculation are met.
    """

    calculation: str
    """A logical expression that that is evaluated to a Boolean.

    If it evaluates as `True`, a Notification for the Event is created and sent to
    the configured destination. Calculations can reference numeric, string, and
    boolean Event fields.

    See
    [Creating Calculations](https://www.m3ter.com/docs/guides/utilizing-events-and-notifications/key-concepts-and-relationships#creating-calculations)
    in the m3ter documentation for more information.
    """

    version: int
    """The version number for the Notification:

    - **Create:** Not valid for initial insertion of new entity - _do not use for
      Create_. On initial Create, version is set at 1 and listed in the response.
    - **Update:** On Update, version is required and must match the existing version
      because a check is performed to ensure sequential versioning is preserved.
      Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import debit_reason_list_params, debit_reason_create_params, debit_reason_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.debit_reason_response import DebitReasonResponse

__all__ = ["DebitReasonsResource", "AsyncDebitReasonsResource"]


class DebitReasonsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DebitReasonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DebitReasonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DebitReasonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return DebitReasonsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """Create a new Debit Reason for your Organization.

        When you've created a Debit
        Reason, it becomes available as a debit type for adding Debit line items to
        Bills. See [Debits](https://www.m3ter.com/docs/api#tag/Debits).

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/picklists/debitreasons",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                debit_reason_create_params.DebitReasonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Retrieve the Debit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Update the Debit Reason with the given UUID.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                debit_reason_update_params.DebitReasonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[DebitReasonResponse]:
        """Retrieve a list of the Debit Reason entities created for your Organization.

        You
        can filter the list returned for the call by Debit Reason ID, Debit Reason short
        code, or by Archive status.

        Args:
          archived: Filter using the boolean archived flag. DebitReasons can be archived if they are
              obsolete.

              - TRUE includes DebitReasons that have been archived.
              - FALSE excludes archived DebitReasons.

          codes: List of Debit Reason short codes to retrieve.

          ids: List of Debit Reason IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of Debit Reasons to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/debitreasons",
            page=SyncCursor[DebitReasonResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    debit_reason_list_params.DebitReasonListParams,
                ),
            ),
            model=DebitReasonResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Delete the Debit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )


class AsyncDebitReasonsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDebitReasonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDebitReasonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDebitReasonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncDebitReasonsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """Create a new Debit Reason for your Organization.

        When you've created a Debit
        Reason, it becomes available as a debit type for adding Debit line items to
        Bills. See [Debits](https://www.m3ter.com/docs/api#tag/Debits).

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/picklists/debitreasons",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                debit_reason_create_params.DebitReasonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Retrieve the Debit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        archived: bool | NotGiven = NOT_GIVEN,
        code: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Update the Debit Reason with the given UUID.

        Args:
          name: The name of the entity.

          archived: A Boolean TRUE / FALSE flag indicating whether the entity is archived. An entity
              can be archived if it is obsolete.

              - TRUE - the entity is in the archived state.
              - FALSE - the entity is not in the archived state.

          code: The short code for the entity.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "archived": archived,
                    "code": code,
                    "version": version,
                },
                debit_reason_update_params.DebitReasonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        archived: bool | NotGiven = NOT_GIVEN,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[DebitReasonResponse, AsyncCursor[DebitReasonResponse]]:
        """Retrieve a list of the Debit Reason entities created for your Organization.

        You
        can filter the list returned for the call by Debit Reason ID, Debit Reason short
        code, or by Archive status.

        Args:
          archived: Filter using the boolean archived flag. DebitReasons can be archived if they are
              obsolete.

              - TRUE includes DebitReasons that have been archived.
              - FALSE excludes archived DebitReasons.

          codes: List of Debit Reason short codes to retrieve.

          ids: List of Debit Reason IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of Debit Reasons to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/picklists/debitreasons",
            page=AsyncCursor[DebitReasonResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    debit_reason_list_params.DebitReasonListParams,
                ),
            ),
            model=DebitReasonResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DebitReasonResponse:
        """
        Delete the Debit Reason with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/picklists/debitreasons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DebitReasonResponse,
        )


class DebitReasonsResourceWithRawResponse:
    def __init__(self, debit_reasons: DebitReasonsResource) -> None:
        self._debit_reasons = debit_reasons

        self.create = to_raw_response_wrapper(
            debit_reasons.create,
        )
        self.retrieve = to_raw_response_wrapper(
            debit_reasons.retrieve,
        )
        self.update = to_raw_response_wrapper(
            debit_reasons.update,
        )
        self.list = to_raw_response_wrapper(
            debit_reasons.list,
        )
        self.delete = to_raw_response_wrapper(
            debit_reasons.delete,
        )


class AsyncDebitReasonsResourceWithRawResponse:
    def __init__(self, debit_reasons: AsyncDebitReasonsResource) -> None:
        self._debit_reasons = debit_reasons

        self.create = async_to_raw_response_wrapper(
            debit_reasons.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            debit_reasons.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            debit_reasons.update,
        )
        self.list = async_to_raw_response_wrapper(
            debit_reasons.list,
        )
        self.delete = async_to_raw_response_wrapper(
            debit_reasons.delete,
        )


class DebitReasonsResourceWithStreamingResponse:
    def __init__(self, debit_reasons: DebitReasonsResource) -> None:
        self._debit_reasons = debit_reasons

        self.create = to_streamed_response_wrapper(
            debit_reasons.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            debit_reasons.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            debit_reasons.update,
        )
        self.list = to_streamed_response_wrapper(
            debit_reasons.list,
        )
        self.delete = to_streamed_response_wrapper(
            debit_reasons.delete,
        )


class AsyncDebitReasonsResourceWithStreamingResponse:
    def __init__(self, debit_reasons: AsyncDebitReasonsResource) -> None:
        self._debit_reasons = debit_reasons

        self.create = async_to_streamed_response_wrapper(
            debit_reasons.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            debit_reasons.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            debit_reasons.update,
        )
        self.list = async_to_streamed_response_wrapper(
            debit_reasons.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            debit_reasons.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageGetFailedIngestDownloadURLParams"]


class UsageGetFailedIngestDownloadURLParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    file: str
    """The file path"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["NotificationConfigurationListParams"]


class NotificationConfigurationListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    active: bool
    """
    A Boolean flag indicating whether to retrieve only active or only inactive
    Notifications.

    - **TRUE** - only active Notifications are returned.
    - **FALSE** - only inactive Notifications are returned.
    """

    event_name: Annotated[str, PropertyInfo(alias="eventName")]
    """
    Use this to filter the Notifications returned - only those Notifications that
    are based on the _Event type_ specified by `eventName` are returned.
    """

    ids: List[str]
    """A list of specific Notification UUIDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Notifications in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of Notifications to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import date, datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ContractResponse"]


class ContractResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """The unique identifier (UUID) of the Account associated with this Contract."""

    code: Optional[str] = None
    """The short code of the Contract."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this Contract."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    description: Optional[str] = None
    """The description of the Contract, which provides context and information."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the Contract was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO-8601 format)_ when the Contract was last modified."""

    end_date: Optional[date] = FieldInfo(alias="endDate", default=None)
    """The exclusive end date of the Contract _(in ISO-8601 format)_.

    This means the Contract is active until midnight on the day **_before_** this
    date.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this Contract."""

    name: Optional[str] = None
    """The name of the Contract."""

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)
    """The Purchase Order Number associated with the Contract."""

    start_date: Optional[date] = FieldInfo(alias="startDate", default=None)
    """The start date for the Contract _(in ISO-8601 format)_.

    This date is inclusive, meaning the Contract is active from this date onward.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["IntegrationConfigurationResponse"]


class IntegrationConfigurationResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    destination: str
    """The destination system for the integration."""

    entity_id: str = FieldInfo(alias="entityId")
    """The unique identifier (UUID) of the entity the integration is for."""

    entity_type: str = FieldInfo(alias="entityType")
    """The type of entity the integration is for _(e.g. Bill)_."""

    status: Literal[
        "WAITING",
        "STARTED",
        "COMPLETE",
        "ERROR",
        "AWAITING_RETRY",
        "AUTH_FAILED",
        "ACCOUNTING_PERIOD_CLOSED",
        "INVOICE_ALREADY_PAID",
        "DISABLED",
    ]

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    dt_completed: Optional[datetime] = FieldInfo(alias="dtCompleted", default=None)
    """The date and time the integration was completed _(in ISO-8601 format)_."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    dt_started: Optional[datetime] = FieldInfo(alias="dtStarted", default=None)
    """The date and time the integration was started _(in ISO-8601 format)_."""

    error: Optional[str] = None
    """Describes any errors encountered during the integration run."""

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """The external ID in the destination system if available."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    url: Optional[str] = None
    """The URL of the entity in the destination system if available."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BillListParams"]


class BillListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """Optional filter.

    An Account ID - returns the Bills for the single specified Account.
    """

    bill_date: Annotated[str, PropertyInfo(alias="billDate")]
    """The specific date in ISO 8601 format for which you want to retrieve Bills."""

    bill_date_end: Annotated[str, PropertyInfo(alias="billDateEnd")]
    """Only include Bills with bill dates earlier than this date."""

    bill_date_start: Annotated[str, PropertyInfo(alias="billDateStart")]
    """Only include Bills with bill dates equal to or later than this date."""

    billing_frequency: Annotated[Optional[str], PropertyInfo(alias="billingFrequency")]

    exclude_line_items: Annotated[bool, PropertyInfo(alias="excludeLineItems")]
    """Exclude Line Items"""

    external_invoice_date_end: Annotated[str, PropertyInfo(alias="externalInvoiceDateEnd")]
    """Only include Bills with external invoice dates earlier than this date."""

    external_invoice_date_start: Annotated[str, PropertyInfo(alias="externalInvoiceDateStart")]
    """
    Only include Bills with external invoice dates equal to or later than this date.
    """

    ids: List[str]
    """Optional filter. The list of Bill IDs to retrieve."""

    include_bill_total: Annotated[bool, PropertyInfo(alias="includeBillTotal")]
    """Include Bill Total"""

    locked: bool
    """Boolean flag specifying whether to include Bills with "locked" status.

    - **TRUE** - the list inlcudes "locked" Bills.
    - **FALSE** - excludes "locked" Bills from the list.
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Bills in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Specifies the maximum number of Bills to retrieve per page."""

    status: Literal["PENDING", "APPROVED"]
    """Only include Bills having the given status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.currency_conversion import CurrencyConversion

__all__ = ["OrganizationConfigUpdateParams"]


class OrganizationConfigUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    currency: Required[str]
    """The currency code for the Organization. For example: USD, GBP, or EUR:

    - This defines the _billing currency_ for the Organization. You can override
      this by selecting a different billing currency at individual Account level.
    - You must first define the currencies you want to use in your Organization. See
      the [Currency](https://www.m3ter.com/docs/api#tag/Currency) section in this
      API Reference.

    **Note:** If you use a different currency as the _pricing currency_ for Plans to
    set charge rates for Product consumption by an Account, you must define a
    currency conversion rate from the pricing currency to the billing currency
    before you run billing for the Account, otherwise billing will fail. See below
    for the `currencyConversions` request parameter.
    """

    day_epoch: Required[Annotated[str, PropertyInfo(alias="dayEpoch")]]
    """
    Optional setting that defines the billing cycle date for Accounts that are
    billed daily. Defines the date of the first Bill:

    - For example, suppose the Plan you attach to an Account is configured for daily
      billing frequency and will apply to the Account from January 1st, 2022 until
      June 30th, 2022. If you set a `dayEpoch` date of January 2nd, 2022, then the
      first Bill is created for the Account on that date and subsequent Bills are
      created for the Account each day following through to the end of the billing
      service period.
    - The date is in ISO-8601 format.
    """

    days_before_bill_due: Required[Annotated[int, PropertyInfo(alias="daysBeforeBillDue")]]
    """
    Enter the number of days after the Bill generation date that you want to show on
    Bills as the due date.

    **Note:** If you define `daysBeforeBillDue` at individual Account level, this
    will take precedence over any `daysBeforeBillDue` setting defined at
    Organization level.
    """

    month_epoch: Required[Annotated[str, PropertyInfo(alias="monthEpoch")]]
    """
    Optional setting that defines the billing cycle date for Accounts that are
    billed monthly. Defines the date of the first Bill and then acts as reference
    for when subsequent Bills are created for the Account:

    - For example, suppose the Plan you attach to an Account is configured for
      monthly billing frequency and will apply to the Account from January 1st, 2022
      until June 30th, 2022. If you set a `monthEpoch` date of January 15th, 2022,
      then the first Bill is created for the Account on that date and subsequent
      Bills are created for the Account on the 15th of each month following through
      to the end of the billing service period - February 15th, March 15th, and so
      on.
    - The date is in ISO-8601 format.
    """

    timezone: Required[str]
    """Sets the timezone for the Organization."""

    week_epoch: Required[Annotated[str, PropertyInfo(alias="weekEpoch")]]
    """
    Optional setting that defines the billing cycle date for Accounts that are
    billed weekly. Defines the date of the first Bill and then acts as reference for
    when subsequent Bills are created for the Account:

    - For example, suppose the Plan you attach to an Account is configured for
      weekly billing frequency and will apply to the Account from January 1st, 2022
      until June 30th, 2022. If you set a `weekEpoch` date of January 15th, 2022,
      which falls on a Saturday, then the first Bill is created for the Account on
      that date and subsequent Bills are created for the Account on Saturday of each
      week following through to the end of the billing service period.
    - The date is in ISO-8601 format.
    """

    year_epoch: Required[Annotated[str, PropertyInfo(alias="yearEpoch")]]
    """
    Optional setting that defines the billing cycle date for Accounts that are
    billed yearly. Defines the date of the first Bill and then acts as reference for
    when subsequent Bills are created for the Account:

    - For example, suppose the Plan you attach to an Account is configured for
      yearly billing frequency and will apply to the Account from January 1st, 2022
      until January 15th, 2028. If you set a `yearEpoch` date of January 1st, 2023,
      then the first Bill is created for the Account on that date and subsequent
      Bills are created for the Account on January 1st of each year following
      through to the end of the billing service period - January 1st, 2023, January
      1st, 2024 and so on.
    - The date is in ISO-8601 format.
    """

    auto_approve_bills_grace_period: Annotated[int, PropertyInfo(alias="autoApproveBillsGracePeriod")]
    """Grace period before bills are auto-approved.

    Used in combination with `autoApproveBillsGracePeriodUnit` parameter.

    **Note:** When used in combination with `autoApproveBillsGracePeriodUnit`
    enables auto-approval of Bills for Organization, which occurs when the specified
    time period has elapsed after Bill generation.
    """

    auto_approve_bills_grace_period_unit: Annotated[str, PropertyInfo(alias="autoApproveBillsGracePeriodUnit")]
    """Time unit of grace period before bills are auto-approved.

    Used in combination with `autoApproveBillsGracePeriod` parameter. Allowed
    options are MINUTES, HOURS, or DAYS.

    **Note:** When used in combination with `autoApproveBillsGracePeriod` enables
    auto-approval of Bills for Organization, which occurs when the specified time
    period has elapsed after Bill generation.
    """

    auto_generate_statement_mode: Annotated[
        Literal["NONE", "JSON", "JSON_AND_CSV"], PropertyInfo(alias="autoGenerateStatementMode")
    ]
    """
    Specify whether to auto-generate statements once Bills are _approved_ or
    _locked_. It will not auto-generate if a bill is in _pending_ state.

    The default value is **None**.

    - **None**. Statements will not be auto-generated.
    - **JSON**. Statements are auto-generated in JSON format.
    - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.
    """

    bill_prefix: Annotated[str, PropertyInfo(alias="billPrefix")]
    """Prefix to be used for sequential invoice numbers.

    This will be combined with the `sequenceStartNumber`.

    **NOTES:**

    - If you do not define a `billPrefix`, a default will be used in the Console for
      the Bill **REFERENCE** number. This default will concatenate **INV-** with the
      last four characters of the `billId`.
    - If you do not define a `billPrefix`, the Bill response schema for API calls
      that retrieve Bill data will not contain a `sequentialInvoiceNumber`.
    """

    commitment_fee_bill_in_advance: Annotated[bool, PropertyInfo(alias="commitmentFeeBillInAdvance")]
    """
    Boolean setting to specify whether commitments _(prepayments)_ are billed in
    advance at the start of each billing period, or billed in arrears at the end of
    each billing period.

    - **TRUE** - bill in advance _(start of each billing period)_.
    - **FALSE** - bill in arrears _(end of each billing period)_.
    """

    consolidate_bills: Annotated[bool, PropertyInfo(alias="consolidateBills")]
    """Boolean setting to consolidate different billing frequencies onto the same bill.

    - **TRUE** - consolidate different billing frequencies onto the same bill.
    - **FALSE** - bills are not consolidated.
    """

    credit_application_order: Annotated[
        List[Literal["PREPAYMENT", "BALANCE"]], PropertyInfo(alias="creditApplicationOrder")
    ]
    """
    Define the order in which any Prepayment or Balance amounts on Accounts are to
    be drawn-down against for billing. Four options:

    - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
      credit.
    - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
      credit.
    - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
    - `"BALANCE"`. Only draw-down against Balance credit.

    **NOTES:**

    - You can override this Organization-level setting for `creditApplicationOrder`
      at the level of an individual Account.
    - If the Account belongs to a Parent/Child Account hierarchy, then the
      `creditApplicationOrder` settings are not available, and the draw-down order
      defaults always to Prepayment then Balance order.
    """

    currency_conversions: Annotated[Iterable[CurrencyConversion], PropertyInfo(alias="currencyConversions")]
    """Define currency conversion rates from _pricing currency_ to _billing currency_:

    - You can use the `currency` request parameter with this call to define the
      billing currency for your Organization - see above.
    - You can also define a billing currency at the individual Account level and
      this will override the Organization billing currency.
    - A Plan used to set Product consumption charge rates on an Account might use a
      different pricing currency. At billing, charges are calculated in the pricing
      currency and then converted into billing currency amounts to appear on Bills.
      If you haven't defined a currency conversion rate from pricing to billing
      currency, billing will fail for the Account.
    """

    default_statement_definition_id: Annotated[str, PropertyInfo(alias="defaultStatementDefinitionId")]
    """
    Organization level default `statementDefinitionId` to be used when there is no
    statement definition linked to the account.

    Statement definitions are used to generate bill statements, which are
    informative backing sheets to invoices.
    """

    external_invoice_date: Annotated[str, PropertyInfo(alias="externalInvoiceDate")]
    """Date to use for the invoice date.

    Allowed values are `FIRST_DAY_OF_NEXT_PERIOD` or `LAST_DAY_OF_ARREARS`.
    """

    minimum_spend_bill_in_advance: Annotated[bool, PropertyInfo(alias="minimumSpendBillInAdvance")]
    """
    Boolean setting to specify whether minimum spend amounts are billed in advance
    at the start of each billing period, or billed in arrears at the end of each
    billing period.

    - **TRUE** - bill in advance _(start of each billing period)_.
    - **FALSE** - bill in arrears _(end of each billing period)_.
    """

    scheduled_bill_interval: Annotated[float, PropertyInfo(alias="scheduledBillInterval")]
    """Sets the required interval for updating bills.

    It is an optional parameter that can be set as:

    - **For portions of an hour (minutes)**. Two options: **0.25** (15 minutes) and
      **0.5** (30 minutes).
    - **For full hours.** Enter **1** for every hour, **2** for every two hours, and
      so on. Eight options: **1**, **2**, **3**, **4**, **6**, **8**, **12**, or
      **24**.
    - **Default.** The default is **0**, which disables scheduling.
    """

    sequence_start_number: Annotated[int, PropertyInfo(alias="sequenceStartNumber")]
    """The starting number to be used for sequential invoice numbers.

    This will be combined with the `billPrefix`.

    For example, if you define `billPrefix` to be **INVOICE-** and you set the
    `seqenceStartNumber` as **100**, the first Bill created after updating your
    Organization Configuration will have a `sequentialInvoiceNumber` assigned of
    **INVOICE-101**. Subsequent Bills created will be numbered in time sequence for
    their initial creation date/time.
    """

    standing_charge_bill_in_advance: Annotated[bool, PropertyInfo(alias="standingChargeBillInAdvance")]
    """
    Boolean setting to specify whether the standing charge is billed in advance at
    the start of each billing period, or billed in arrears at the end of each
    billing period.

    - **TRUE** - bill in advance _(start of each billing period)_.
    - **FALSE** - bill in arrears _(end of each billing period)_.
    """

    suppressed_empty_bills: Annotated[bool, PropertyInfo(alias="suppressedEmptyBills")]
    """Boolean setting that supresses generating bills that have no line items.

    - **TRUE** - prevents generating bills with no line items.
    - **FALSE** - bills are still generated even when they have no line items.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.pricing_band import PricingBand

__all__ = ["CounterPricingResponse"]


class CounterPricingResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    accounting_product_id: Optional[str] = FieldInfo(alias="accountingProductId", default=None)

    code: Optional[str] = None
    """Unique short code for the Pricing."""

    counter_id: Optional[str] = FieldInfo(alias="counterId", default=None)
    """UUID of the Counter used to create the pricing."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created this item."""

    cumulative: Optional[bool] = None
    """
    Controls whether or not charge rates under a set of pricing bands configured for
    a Pricing are applied according to each separate band or at the highest band
    reached.

    The default value is **TRUE**.

    - When TRUE, at billing charge rates are applied according to each separate
      band.

    - When FALSE, at billing charge rates are applied according to highest band
      reached.
    """

    description: Optional[str] = None
    """Displayed on Bill line items."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when this item was created _(in ISO-8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when this item was last modified _(in ISO-8601 format)_."""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """
    The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
    the Plan or Plan Template.

    If not specified, the Pricing remains active indefinitely.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified this item."""

    plan_id: Optional[str] = FieldInfo(alias="planId", default=None)
    """UUID of the Plan the Pricing is created for."""

    plan_template_id: Optional[str] = FieldInfo(alias="planTemplateId", default=None)
    """UUID of the Plan Template the Pricing was created for."""

    pricing_bands: Optional[List[PricingBand]] = FieldInfo(alias="pricingBands", default=None)

    pro_rate_adjustment_credit: Optional[bool] = FieldInfo(alias="proRateAdjustmentCredit", default=None)
    """The default value is **TRUE**.

    - When TRUE, counter adjustment credits are prorated and are billed according to
      the number of days in billing period.

    - When FALSE, counter adjustment credits are not prorated and are billed for the
      entire billing period.
    """

    pro_rate_adjustment_debit: Optional[bool] = FieldInfo(alias="proRateAdjustmentDebit", default=None)
    """The default value is **TRUE**.

    - When TRUE, counter adjustment debits are prorated and are billed according to
      the number of days in billing period.

    - When FALSE, counter adjustment debits are not prorated and are billed for the
      entire billing period.
    """

    pro_rate_running_total: Optional[bool] = FieldInfo(alias="proRateRunningTotal", default=None)
    """The default value is **TRUE**.

    - When TRUE, counter running total charges are prorated and are billed according
      to the number of days in billing period.

    - When FALSE, counter running total charges are not prorated and are billed for
      the entire billing period.
    """

    running_total_bill_in_advance: Optional[bool] = FieldInfo(alias="runningTotalBillInAdvance", default=None)
    """The default value is **TRUE**.

    - When TRUE, running totals are billed at the start of each billing period.

    - When FALSE, running totals are billed at the end of each billing period.
    """

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """
    The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
    for the Plan of Plan Template.
    """
